# EGN-EP FLOWCHARTS

## OVERVIEW

```mermaid
flowchart TD
    p0((genome))
    p1((parameters))
    p2((proteinDBs))
    p3((transcriptDBs))
    p4((repeatDBs))
    p10(abInitio\nWorkflow)
    p20(proteins\nWorkflow)
    p30(transcripts\nWorkflow)
    p40(training\nWorkflow)
    p10 --> p40 --> p50
    p20 --> p40 --> p80
    p30 --> p40
    p4 --> p50
    p50(speciesRepeatsPrediction)
    p60(repeatsScan)
    p70(genomeMasking\nWorkflow)
    p80(annotation\nWorkflow)
    p200((genome\nAnnotation))
    p300((statistics))
    p0 --> p10 --> p80
    p1 --> p10
    p0 --> p20 --> p80 --> p300
    p2 --> p20
    p0 --> p30 --> p80 --> p200
    p3 --> p30
    p50 --> p60 --> p70 --> p80
```
## AB-INITIO WORKFLOW

```mermaid
flowchart TD
    classDef parameters fill:#f96;
    p0((genome))
    p3{Distribute}
    p4{Distribute}
    p100((abInitio\nEvidences))
    p8[buildOverlappingSequences:LIPMUTILS_fasta2overlappingwins]
    p9[buildOverlappingSequences:LIPMUTILS_fasta2tree]
    p13[buildLargeOverlappingSequences:LIPMUTILS_fasta2overlappingwins]
    p14[buildLargeOverlappingSequences:LIPMUTILS_fasta2tree]
    p0 --> p8:::parameters --> p9 --> p3
    p0 --> p13:::parameters --> p14 --> p4
    p17[GENOMETOOLS_ltrharvest]
    p20[EGNPCORE_changeGff3CoordinatesBatch]
    p21[GENOMETOOLS_sortGff3]
    p28[EGNPCORE_PublishEvidences_ltrharvest]
    p4 --> p17:::parameters --> p20 --> p21 --> p28
    p30[RED_scan]
    p32[BEDTOOLS_mergeBed]
    p34[BEDTOOLS_minLengthFilter]
    p35[LIPMUTILS_splitBed]
    p40[LIPMUTILS_bedToGff3]
    p47[EGNPCORE_PublishEvidences_red]
    p0 --> p30:::parameters --> p32:::parameters --> p34:::parameters --> p35 --> p40 --> p47
    p51[TRNASCANSE_trnascanSE]
    p54[EGNPCORE_changeGff3CoordinatesBatch]
    p55[GENOMETOOLS_sortGff3]
    p57[EGNPCORE_splitByStrand]
    p88[EGNPCORE_PublishEvidences_trnascanse]
    p4 --> p51:::parameters --> p54 --> p55 --> p57 --> p88
    p61[BARRNAP_barrnap]
    p64[EGNPCORE_changeGff3CoordinatesBatch]
    p65[GENOMETOOLS_sortGff3]
    p67[EGNPCORE_splitByStrand]
    p97[EGNPCORE_PublishEvidences_barrnap]
    p4 --> p61:::parameters --> p64 --> p65 --> p67 --> p97
    p71[INFERNAL_rfamscan]
    p74[EGNPCORE_changeGff3CoordinatesBatch]
    p75[GENOMETOOLS_sortGff3]
    p77[EGNPCORE_splitByStrand]
    p106[EGNPCORE_PublishEvidences_rfamscan]
    p3 --> p71:::parameters --> p74 --> p75 --> p77 --> p106
    p106 --> p100
    p97 --> p100
    p88 --> p100
    p28 --> p100
    p47 --> p100
```

Parameters:

```java
    windowMaxLen            = 2000000
    windowOverlapLen        = 10000
    /*for fast steps (barrnap, trnascanse, ltrharvest, blatx repeats)
     multiply windowMaxLen by this value */
    largeWindowFactor       = 50
  
    //SKIP STEPS
    skip_trnascan           = false
    skip_rfamscan           = false
    skip_barrnap            = false
    skip_ncrna_detection    = false
    skip_repeat_masking     = false
    skip_red                = false

    //RED param
    red_param               = '-len 16 -frm 2 -min 6'
    red_minlen              = 500
    red_mergingdistance     = 100
```
## PROTEINS WORKFLOW

```mermaid
flowchart TD
	classDef parameters fill:#f96;
    p0((genome))
    p2((proteinDB\nconf))
    p100((protein\nEvidences))
    p4[LIPMUTILS_fasta2overlappingwins]
    p5[LIPMUTILS_fasta2tree]
    p0 --> p4:::parameters --> p5 --> p18
    p13[BLASTX_indexProtDB]
    p16[BLASTX_cleanProtDB]
    p2 --> p13 --> p17
    p2 --> |NotCleaned| p16:::parameters --> p17 --> p18
    p17([BlastXDBs])
    p18{Distribute}
    p20[BLASTX_alignHigh]
    p23[BLASTX_alignMedium]
    p27[BLASTX_alignLow]
    p18 --> |big Proteome| p20:::parameters --> p31
    p18 --> |medium Proteome| p23:::parameters --> p31
    p18 --> |small Proteome| p27:::parameters --> p31
    p31[EGNPCORE_changeBlastxCoordinatesBatch]
    p35[LIPMUTILS_m8ToGff3]
    p36[BLASTX_postProcessing]
    p42[EGNPCORE_PublishEvidences]
    p31 --> p35 --> p36:::parameters --> p42 --> p100
```

Parameters:

```java  
windowMaxLen           = 2000000
windowOverlapLen     = 10000

dbsize_switch_resource_high   = 500000000
dbsize_switch_resource_low    = 50000000

/* Method for the protein similarity search. Allowed values are:
    - ublast_blastx  First perform a protein database reduction with ublast from usearch.
    - diamond_blastx First perform a protein database reduction with diamond. 
 Then launch classical blastx search against the reduced db.
*/

protein_similarity_search_method = 'diamond_blastx'
blastx_param = '-outfmt 6 -evalue 0.000001 -gapopen 9 -gapextend 2 -max_target_seqs 500000 
                -max_intron_length 15000  -seg yes'
blastx_unique_filter = true
split_blastx_db      = true

// Used by targetedblastsearch program
prg_targetedblastsearch_param = '--db_length 1000000000'
prg_usearch_ublast_param      = '-evalue 1 -lopen 9 -lext 2 -accel 1'
prg_diamond_param             = '--sensitive --evalue 1 --gapopen 9 --gapextend 2 --masking 0
                                 --comp-based-stats 0 --block-size 0.4'

```
## TRANSCRIPTS WORKFLOW

```mermaid
flowchart TD
	classDef parameters fill:#f96;
    p0((genome))
    p2((transcripts\nConf))
    p5[GMAP_index]
    p0 --> p5
    p6{Distribute}
    p7[LIPMUTILS_trimPolyX]
    p100((transcripts\nEvidences))
    p2 --> p7 --> p10 --> p6
    p7 --> p11 --> p6
    p5 --> p6
    p10[EGNPCORE_dumpEstCfg]
    p11[LIPMUTILS_splitEstDBs]
    p18[GMAP_map]
    p21[GMAP_concat]
    p24[GMAP_concatRmSpliced]
    p29[EGNPCORE_splitGff3BySeqid]
    p32[EGNPCORE_splitGff3BySeqidRmSpliced]
    p37[GMAP_postProcessing]
    p39[GMAP_reportSummary]
    p42[GMAP_updConf]
    p49[EGNPCORE_PublishEvidences]
    p6 --> p18:::parameters --> p21 --> p29 --> p37:::parameters
    p18 --> p24 --> p32 --> p37
    p10 --> p37 --> p39
    p37 --> p42
    p37 --> p49 --> p100
```

Parameters:

```java
est_num_per_slice            = 50000
gmap_smallexons_minlen       = 25
gmap_param                   = "-n0 -B 5 -L 100000 --min-intronlength=35 -K 25000 
                                --trim-end-exons=${params.gmap_smallexons_minlen}"
gmap_MIN_LEN_SHORT_UNSPLICED = 10000000
gmap_intron_filter           = true
gmap_unique_filter           = true
//FILTER WEIRD ALN 
gmap_filter_min_exon_len     = 10
gmap_filter_max_short_exon_number = 1
allestnb                     = 1000
/*  0: same weight to all alignments; 
    1: unspliced alignments ignored; 
    2: more weight is given to the spliced alignments */
allest_remove_unspliced      = 2
```
## TRAINING WORKFLOW

```mermaid
flowchart TD
    classDef parameters fill:#f96;
    p0((genome))
    p2((raw\nGmap\nResults))
    p3((reference\nProteome))
    p4((reference\nTranscriptome))
    p5((matrix))
    p6{Distribute}
    p7((Non\nCanonical\nConf))
    p52[EGNPCORE_selectTranscriptsPerSlice]
    p56[EGNPCORE_ConcatGff3]
    p59[EGNPCORE_selectTranscripts]
    p2 --> p52:::parameters --> p56 --> p59:::parameters --> p67:::parameters --> p71 --> p6 --> p79
    p67[LIPMUTILS_fastaFilterIdsAndMinLen]
    p71[LIPMUTILS_splitSingleDB]
    p76[BLAST_makeblastdb]
    p79[BLASTX_alignForTraining]
    p83[EGNPCORE_filterHspsForExtractCDS]
    p86[EGNPCORE_CatGzip]
    p91[EGNPCORE_extractCDSFromBlastx]
    p98[GMAP_map]
    p103[GMAP_concat]
    p106[EGNPCORE_BuildTrainingDataSets]
    p107[EGNPCORE_DetectNonCanSites]
    p110[EGNPCORE_FastaTotrainIMM]
    p111[EUGENE_trainIMM]
    p3 --> p76 --> p6 
    p79:::parameters --> p83:::parameters --> p86 --> p91 --> |cdsModel|p111 --> p5
    p0 --> p98
    p4 --> p98 --> p103 --> p106 --> p107 --> p7
    p106 --> p110 --> |intronsModel|p111
```

Parameters:

```java
//TRAINING
  training_min_est_mapped = 50
  training_use_gmap_cds = false
  cdhit_cds_identity = 0.99
  cdhit_cds_span = 0.99
  
  
  build_training_dataset_param  = ''
  // Blastx filters  (Blast the reference proteome against the reference trancriptome)
  // hsp_training_length amino acid number!
  hsp_training_splitsize  = 1000
  hsp_training_length     = 100
  hsp_training_pci        = 50
  hsp_training_evalue     = 0.000001
  hsp_training_min_nb     = 300
  hsp_training_blastx_param = '-outfmt 6 -evalue 0.000001 -gapopen 9 -gapextend 2 
                              -max_target_seqs 500000 -max_hsps 2 -max_intron_length 15000 -seg yes'
  
  /* Mapping filters (Map the reference transcriptome to the genome, then filter results)
   Intronic sequences are extracted and used to build intronic IMM models */
  training_est_pcs  = 99
  training_est_pci  = 99
  training_est_remove_unspliced = 1
  
  /* Only use for Full Length (FL) transcriptome (est_priority value >=2)
   EuGene regards the regions flanking FL transcript alignments as intergenic regions.
  FL_flanking_region_length is the length of that regions.*/
  FL_flanking_region_length = 20

  /*SPLICE SITES
   A non canonical splice site is allowed if present more than X percent compared to the canonical sites 
   Default value 1% choosen with Arath training data*/
  noncansite_required_percent = 0.5
  //Maximum number of non canonical spice site detected
  max_noncansite_candidate_nb = 10
  noncanacc = ''
  noncandon = ''

 //EuGene Parameters
 eugene_params = ''
```
## BUILD SPECIES SPECIFIC REPEATS

```mermaid
flowchart TD
    classDef parameters fill:#f96;
    p0((pseudomolecules))
    p1((all\nEvidences))
    p2((all\nTranscripts\nConf))
    p3((Non\nCanonical\nConf))
    p4{Distribute}
    p5{Distribute}
    p258[EGNPCORE_BuildEGNEukParameterFileFW]
    p262[EGNPCORE_BuildEGNEukParameterFileRV]
    p266[EUGENE_runFW]
    p270[EUGENE_runRV]
    p1 --> p4 --> p262 --> p270 --> p273
    p0 --> p266 
    p0 --> p270 
    p2 --> p4 --> p258 --> p266 --> p273
    p3 --> p4
    p273[BIOFILECONVERTER_sortGff3]
    p275[BIOFILECONVERTER_gff2fasta]
    p282[EGNPCORE_ConcatFastaProt0]
    p284[LIPMUTILS_fastaFilterMinLen]
    p287[LIPMUTILS_splitSingleDB]
    p289((Dfam))
    p293[BLAST_makeblastdbDfam]
    p290((TransposonsPSI\nDB))
    p298[BLAST_makeblastdbTransposons]
    p273 --> p275 --> p282 --> p284 --> p287 
    p302[BLAST_extractRepeatDomain]
    p305[EGNPCORE_ConcatFasta]
    p309[EGNPCORE_ConcatFastaAll]
    p310[LIPMUTILS_nrRename]
    p311[CDHIT_speciesRepeats]
    p312[BLASTX_index]
    p287 --> p5 --> p302:::parameters
    p289 --> p293 --> p302:::parameters
    p290 --> p298 --> p302:::parameters
    p290 --> p309
    p302 --> p305 --> p309 --> p310 --> p311:::parameters --> p312 --> p314
    p311 --> p313
    p313((Repeats))
    p314((Repeats\nIndex))
```

Parameters:

```java
//SPECIES SPECIFIC REPEATS
use_repbase = false
prg_remove_repbase_blastparam = ''
prg_extract_repeat_domain_min_scov   =  80
prg_extract_repeat_domain_blastparam = ''
prg_extract_repeat_domain_batchsize = 1000
//Annotation V0 filter
repeat_min_length = 200
cdhit_repeat_identity = 0.7
cdhit_repeat_span = 0.8
//Allocated memory (in Mb)
cdhit_memory = 1024
```
## REPEATS SEARCH

```mermaid
flowchart TD
	classDef parameters fill:#f96;
    p0((pseudomolecules))
    p1((masking\nEvidences))
    p2((Repeats))
    p3((Repeats\nIndex))
    p4{Distribute}
    p5((repeats\nEvidences))
    p318[EGNPCORE_MaskFromLtrHarvest]
    p321[EGNPCORE_MaskFromRed]
    p327[EGNPCORE_ConcatFasta]
    p0 --> p4
    p1 --> p4 --> |ltrharvestEvidences|p318 --> p327
    p4 --> |redEvidences|p321 --> p327
    p2 --> p336
    p3 --> p336
    p327 --> |largeWindows|p329:::parameters --> p330 --> p336:::parameters --> p339 --> p340 --> p5
    p329[LIPMUTILS_fasta2overlappingwins]
    p330[LIPMUTILS_fasta2tree]
    p336[BLASTX_Repeats]
    p339[EGNPCORE_changeBlastxCoordinatesBatch]
    p340[LIPMUTILS_m8RepeatToGff3]
```

Parameters:

```java
/* Method for the protein similarity search. Allowed values are:
    - ublast_blastx  First perform a protein database reduction with ublast from usearch.
    - diamond_blastx First perform a protein database reduction with diamond. 
 Then launch classical blastx search against the reduced db.
*/

protein_similarity_search_method = 'diamond_blastx'
blastx_param = '-outfmt 6 -evalue 0.000001 -gapopen 9 -gapextend 2 -max_target_seqs 500000 
                -max_intron_length 15000  -seg yes'
blastx_unique_filter = true
split_blastx_db      = true

// Used by targetedblastsearch program
prg_targetedblastsearch_param = '--db_length 1000000000'
prg_usearch_ublast_param      = '-evalue 1 -lopen 9 -lext 2 -accel 1'
prg_diamond_param             = '--sensitive --evalue 1 --gapopen 9 --gapextend 2 --masking 0
                                 --comp-based-stats 0 --block-size 0.4'
```
## GENOME MASKING

```mermaid
flowchart TD
    classDef parameters fill:#f96;
    p0((pseudomolecules))
    p1((ncRna\nEvidences))
    p2((transcripts\nEvidences))
    p3((proteins\nEvidences))
    p4((repeats\nEvidences))
    p6{Distribute}
    p7((Masked\nGenome))
    p10[EGNPCORE_mergeRepeatGff3]
    p4 --> p10
    p0 --> p6 --> p10 --> p12
    p6 --> p11 --> p12 --> p13 --> p14 --> p15 --> p7
    p1 --> p11
    p2 --> |preservedDB|p11
    p3 --> |preservedDB|p11
    p11[EGNPCORE_mergePreserveGff3]
    p12[BEDTOOLS_subtract]
    p13[EGNPCORE_setTypeGFF3]
    p14[EGNPCORE_splitByStrand]
    p15[EGNPCORE_PublishEvidences_repeat_noexpressed_nosimprot]
```
## FINAL ANNOTATION

```mermaid
flowchart TD
    classDef parameters fill:#f96;
    p0((pseudomolecules))
    p1((allEvidences))
    p2((allTranscriptsConf))
    p3((NonCanonicalConf))
    p4{Distribute}
    p5((ncRnaEvidences))
    p6((completeGenome))
    p100((FastaFiles))
    p101((Gff3File))
    p102((DiffBedFile))
    p103((DiffSummary))
    p104((Statistics))
    p258[EGNPCORE_BuildEGNEukParameterFileFW]
    p262[EGNPCORE_BuildEGNEukParameterFileRV]
    p1 --> p4 --> p262 --> p270
    p2 --> p4 --> p258 --> p266
    p3 --> p4
    p0 --> p270:::parameters --> p273
    p0 --> p266:::parameters --> p273
    p266[EUGENE_runFW]
    p270[EUGENE_runRV]
    p273[BIOFILECONVERTER_sortGff3]
    p10[LIPMUTILS_transfertGff3Attributes]
    p11[BIOFILECONVERTER_sortGff3Final]
    p12[EGNPCORE_ConcatFinalGff3]
    p13[BIOFILECONVERTER_gff2fasta]
    p14[EGNPCORE_finalPublish]
    p6 --> p13 --> p15 --> p104
    p273 --> p10 --> p11 --> p12 --> p13 --> p14 --> p100
    p14 --> p101
    p13 --> |proteins|p16:::parameters --> p18 --> p103
    p6 --> p17:::parameters --> p18 --> p102
    p6 --> p15
    p5 --> p10
    p15[LIPMUTILS_annotationStatistics]
    p16[BUSCO_protein]
    p17[BUSCO_genome]
    p18[BUSCO_summary]
```

Parameters:
```java
organism		= 		'Genus species'
output_prefix 		=		'myGenome'
locus_tag_prefix	=		'LOCUSTAG'
independent_strand_annotation = true
//Domain: fungi,nematodes,oomycetes,plant
kingdom             = 		'eukaryote'
domain				=		'plant'
//BUSCO (https://busco.ezlab.org/list_of_lineages.html)
busco_lineage_dataset = 'viridiplantae_odb10'
```
