nextflow.enable.dsl=2


process BEDTOOLS_mergeBed {

	//grep exit code = 1 if empty results
    // errorStrategy { task.exitStatus in 0..1 ? 'ignore' : 'terminate' }

    input:
    path(infile)
    val(mergingdistance)
    
	output:
    path("${infile.baseName}.merged.bed")
    
    script:
    """
    bedtools merge -d ${mergingdistance} -i ${infile} >  ${infile.baseName}.merged.bed
    """
}

process BEDTOOLS_minLengthFilter {

    errorStrategy { task.exitStatus in 0..1 ? 'ignore' : 'terminate' }

    input:
    path(infile)
    val(minLen)
    
    output:
    path("${infile.baseName}.filtered.bed")
    
    script:
    """
    if grep -q ^# "${infile}"; then
      grep '^#' ${infile} > ${infile.baseName}.filtered.bed
    else 
      echo "#${infile.baseName}" > ${infile.baseName}.filtered.bed
    fi
    awk '{if(\$3-\$2 >= ${minLen}) print}' ${infile} >> ${infile.baseName}.filtered.bed
    """
}

process BEDTOOLS_subtract {

	input:
    tuple val (seqid), path(infile), path(tosubtract)
    val (stranded)

	output:
    tuple val (seqid), path("${infile.baseName}.subtract")
    
    script:

    strandparam = ""
    if (stranded == true){
        strandparam = " -s "
    }

    """
    bedtools subtract -a ${infile} -b ${tosubtract} ${strandparam} >  ${infile.baseName}.subtract
    [ -s ${infile.baseName}.subtract ] && touch  ${infile.baseName}.subtract || echo "##gff-version 3" >  ${infile.baseName}.subtract
    """
}


process BEDTOOLS_softwaresInfo {
	output:
	path("bedtools.softwares_info.tsv")

	script:
	"""
	softwares_info > bedtools.softwares_info.tsv
	"""
}