nextflow.enable.dsl=2

include { BLASTX_index;BLASTX_Repeats;BLAST_extractRepeatDomain } from '../software/blast/main.nf'
include { BLAST_makeblastdb as BLAST_makeblastdbDfam; BLAST_makeblastdb as BLAST_makeblastdbTransposons; BLAST_makeblastdb as BLAST_makeblastdbRepbase } from '../software/blast/main.nf'
include { EUGENE_run; EUGENE_run as EUGENE_runFW; EUGENE_run as EUGENE_runRV;EUGENE_runBigGenome; EUGENE_runBigGenome as EUGENE_runBigGenomeFW; EUGENE_runBigGenome as EUGENE_runBigGenomeRV  }  from '../software/eugene/main.nf'

include { BIOFILECONVERTER_gff2fasta;BIOFILECONVERTER_sortGff3 } from '../software/biofileconverter/main.nf'
include { EGNPCORE_ConcatFasta;EGNPCORE_ConcatFasta as EGNPCORE_ConcatFastaAll;EGNPCORE_ConcatFasta as EGNPCORE_ConcatFastaProt0 } from '../software/egnpcore/main.nf'
include { EGNPCORE_BuildEGNEukParameterFile;EGNPCORE_changeBlastxCoordinatesBatch } from '../software/egnpcore/main.nf'
include { EGNPCORE_BuildEGNEukParameterFile as EGNPCORE_BuildEGNEukParameterFileFW;EGNPCORE_BuildEGNEukParameterFile as EGNPCORE_BuildEGNEukParameterFileRV } from '../software/egnpcore/main.nf'

include { EGNPCORE_MaskSequence as EGNPCORE_MaskFromLtrHarvest;EGNPCORE_MaskSequence as EGNPCORE_MaskFromRed } from '../software/egnpcore/main.nf'
include { LIPMUTILS_fastaFilterMinLen;LIPMUTILS_m8RepeatToGff3;LIPMUTILS_splitSingleDB;LIPMUTILS_nrRename;LIPMUTILS_fasta2overlappingwins;LIPMUTILS_fasta2tree }  from '../software/lipmutils/main.nf'
include { CDHIT_speciesRepeats } from '../software/cdhit/main.nf'


workflow speciesSpecificRepeats {
	take:
		completeGenome
		pseudomolecules
		configMap
		matrix
		noncan_cfg
		evidencesTuples
		allEstConf
		run_parameters

	main:
	if (configMap.repeat.species_repeat_domains)
	{
		Utils.info("Species specific repeats provided by user",params.verbose)
		repeatFasta = LIPMUTILS_nrRename(configMap.repeat.species_repeat_domains)
		repeatFastaIndex = BLASTX_index(repeatFasta)
	}
	else
	{
		
		pseudomoleculesTuples = pseudomolecules.map{ it -> tuple (it.trim().split('/').last(),it.trim())}
		sourceTuples = pseudomoleculesTuples.combine(evidencesTuples, by: 0)


		if (! params.independent_strand_annotation)	{
			
				annotationV0_param = EGNPCORE_BuildEGNEukParameterFile('annotationV0',0,0,noncan_cfg,allEstConf,run_parameters)
				
				if (configMap.genome_mode == 'big')
				{
					annotationV0gff3Tuples = EUGENE_runBigGenome(annotationV0_param.collect(), matrix.collect(), sourceTuples, 'REPEAT_')
				}
				else
				{
					annotationV0gff3Tuples = EUGENE_run(annotationV0_param.collect(), matrix.collect(), sourceTuples, 'REPEAT_')
				}
		}
		else {
				annotationV0FW_param = EGNPCORE_BuildEGNEukParameterFileFW('annotationV0',0,1,noncan_cfg,allEstConf,run_parameters)
				annotationV0RV_param = EGNPCORE_BuildEGNEukParameterFileRV('annotationV0',0,-1,noncan_cfg,allEstConf,run_parameters)
				
				if (configMap.genomeMode == 'big')
				{
					annotationV1FWgff3Tuples = EUGENE_runBigGenomeFW(annotationV0FW_param.collect(), matrix.collect(), sourceTuples, 'REPEAT.FW')
					annotationV1RVgff3Tuples = EUGENE_runBigGenomeRV(annotationV0RV_param.collect(), matrix.collect(), sourceTuples, 'REPEAT.RV')
				}
				else 
				{
					annotationV0FWgff3Tuples = EUGENE_runFW(annotationV0FW_param.collect(), matrix.collect(), sourceTuples, 'REPEAT.FW')
					annotationV0RVgff3Tuples = EUGENE_runRV(annotationV0RV_param.collect(), matrix.collect(), sourceTuples, 'REPEAT.RV')
				}
				annotationV0gff3Tuples = BIOFILECONVERTER_sortGff3(annotationV0FWgff3Tuples.concat(annotationV0RVgff3Tuples).groupTuple())
		}	
		(annotationV0FastaGene, annotationV0FastaNcrna, annotationV0FastaMrna, annotationV0FastaCds, annotationV0FastaProt ) =  BIOFILECONVERTER_gff2fasta(pseudomoleculesTuples.combine(annotationV0gff3Tuples, by:0))
		
		//concat annotationV0FastaProt puis split en X slice equilibrees
		annotationV0FastaProtAll = EGNPCORE_ConcatFastaProt0(annotationV0FastaProt.collect(),'prot0')
		//Filtre taille min prot
		annotationV0FastaProtAllFiltered = LIPMUTILS_fastaFilterMinLen(annotationV0FastaProtAll, params.repeat_min_length)
		annotationV0FastaProtSplit = LIPMUTILS_splitSingleDB('prot0', annotationV0FastaProtAllFiltered, params.prg_extract_repeat_domain_batchsize).flatten()

		repeatIndices = Channel.empty();
		repeatConf = configMap['repeat']
		if (params.use_repbase){
			if (! repeatConf['repeat_sequence_db'] ) { exit 1, 'repeat_sequence_db path not specified!' }
			repeatIndices = repeatIndices.concat(BLAST_makeblastdbRepbase(repeatConf['repeat_sequence_db'], 'prot',''))
		}
		else {
			if (! repeatConf['dfam_consensus_db'] ) { exit 1, 'dfam_consensus_db path not specified!' }
			repeatIndices = repeatIndices.concat(BLAST_makeblastdbDfam(repeatConf['dfam_consensus_db'], 'nucl',''))
			if (! repeatConf['transposonpsi_db'] ) { exit 1, 'transposonpsi_db path not specified!' }
			repeatIndices = repeatIndices.concat(BLAST_makeblastdbTransposons(repeatConf['transposonpsi_db'], 'prot',''))
		}

		(speciesRepeatDomainSlice,speciesRepeatDomainWithRepBaseSclice)  = BLAST_extractRepeatDomain(annotationV0FastaProtSplit,configMap['repeat'], repeatIndices.collect())
		speciesRepeatDomain = EGNPCORE_ConcatFasta(speciesRepeatDomainSlice.collect(),'repeat')
		speciesRepeatDomainWithRepBase = EGNPCORE_ConcatFastaAll(speciesRepeatDomainWithRepBaseSclice.collect(),'allrepeat')
		repeatFastaNR = LIPMUTILS_nrRename(speciesRepeatDomainWithRepBase)
		//CDHIT
		repeatFasta = CDHIT_speciesRepeats(repeatFastaNR)
		repeatFastaIndex = BLASTX_index(repeatFasta)

	}

	emit:
	repeatFasta
	repeatFastaIndex

}

workflow searchRepeat {

	take:
	completeGenome
	pseudomolecules
	maskingEvidences
	repeatFasta
	repeatFastaIndex

	main:
	ltrharvestTuples = maskingEvidences.filter{ it.get(1).name.contains(".ltrharvest.") }
	redTuples = maskingEvidences.filter{ it.get(1).name.contains(".red.") }
	pseudomoleculesTuples = pseudomolecules.map{ it -> tuple (it.trim().split('/').last(),it.trim())}


	maskedLTRSeqs = EGNPCORE_MaskFromLtrHarvest(pseudomoleculesTuples.join(ltrharvestTuples,remainder: true), 'ltrharvest')
	maskedRedSeqs = EGNPCORE_MaskFromRed(maskedLTRSeqs.join(redTuples,remainder: true),'ltrharvest-red')
	maskedRedSeqsFiles = maskedRedSeqs.groupTuple().map{it.get(1)}.flatten()
	maskedGenomeFile = EGNPCORE_ConcatFasta(maskedRedSeqsFiles.collect(),'masked')
	windows = LIPMUTILS_fasta2overlappingwins(maskedGenomeFile, params.largeWindowFactor)
	(sequencesFof, tt) = LIPMUTILS_fasta2tree(windows)
	overlappingMaskedSequenceCh = sequencesFof.splitText()
	overlappingMaskedSequenceTuples = overlappingMaskedSequenceCh.map{ it -> tuple (it.trim().split('/').last(),it.trim())}
	blastxRepeats = BLASTX_Repeats(overlappingMaskedSequenceTuples, repeatFasta.collect(), repeatFastaIndex.collect())
	outFilesTuples = blastxRepeats.map {
			t -> 
			def seqid = t.get(0).split(":").first()
			def source = t.get(1)
			def file = t.get(3)
			tuple (seqid,source, file)
		}
		
	blastxRepeatsRelocateM8Tuples = EGNPCORE_changeBlastxCoordinatesBatch(outFilesTuples.groupTuple(by:[0,1]))
	repeatFiles = LIPMUTILS_m8RepeatToGff3(blastxRepeatsRelocateM8Tuples)
	repeatEvidences = maskingEvidences.concat(repeatFiles).groupTuple()

	emit:
	repeatEvidences
	

}
